﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        static SerialPort USB_PORT;
        byte[] SerBuf = new byte[512];
        byte usb_found;
        byte selected_servo = 0;

        public Form1()
        {
            InitializeComponent();
            USB_PORT = new SerialPort();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            int servo = 0;      // Fill the servo selection box with servo numbers
            for (servo = 1; servo <= 84; servo++)
            {
                comboBox_servo.Items.Add(servo);
            }
            
        }

        private void transmit(byte write_bytes)
        {
            try
            {
                if (usb_found == 1) USB_PORT.Write(SerBuf, 0, write_bytes);      // writes specified amount of SerBuf out on serial port
            }
            catch (Exception)
            {
                usb_found = 0;
                MessageBox.Show("write fail");
                return;
            }
        }

        private void receive(byte read_bytes)
        {
            byte x;
            for (x = 0; x < read_bytes; x++)       // this will call the read function for the passed number times, 
            {                                      // this way it ensures each byte has been correctly recieved while
                try                                // still using timeouts
                {
                    if (usb_found == 1) USB_PORT.Read(SerBuf, x, 1);     // retrieves 1 byte at a time and places in SerBuf at position x
                }
                catch (Exception)                   // timeout or other error occured, set lost comms indicator
                {
                    usb_found = 0;
                    MessageBox.Show("read fail");
                    return;
                }
            }
        }

        private void comboBox_comport_MouseDown(object sender, MouseEventArgs e)
        {
            // Grab the list of available Serial ports and place them in the combo box
            comboBox_comport.Items.Clear();
            string[] comportnames = new string[100];
            comportnames = SerialPort.GetPortNames();
            foreach (string port in comportnames)
            {
                // Our serial port will be either a ttyACMx or ttyUSBx device. Filter all such ports
                // out and place them in the combo box.
                if (port.Contains("ACM") || port.Contains("USB")) {
                    comboBox_comport.Items.Add(port);
                }
            }
        }

        private void servo_selected(object sender, EventArgs e)
        {
            selected_servo = byte.Parse(comboBox_servo.SelectedItem.ToString());    // Index of selected item is the number of the servo we wish to control
        }

        private void comboBox_comport_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (USB_PORT.IsOpen) USB_PORT.Close(); // close any existing handle
            }
            catch
            {
                MessageBox.Show("Error closing to serial port", "SD84", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                Application.Exit();
            }
            try
            {
                USB_PORT.PortName = comboBox_comport.Text;    // retrieves serial port from selection in combo box
                // If the serial port name contains "USB" then the port is using an FTDI chip.
                // FTDI chips need the serial port baud rate, pairity, stop bits and data bits defining. 
                if (comboBox_comport.Text.Contains("USB"))
                {
                    USB_PORT.Parity = 0;
                    USB_PORT.BaudRate = 115200;
                    USB_PORT.StopBits = StopBits.Two;
                    USB_PORT.DataBits = 8;
                }
                USB_PORT.ReadTimeout = 500;
                USB_PORT.WriteTimeout = 500;
                USB_PORT.Open();
                usb_found = 1;

                SerBuf[0] = 0xAA;                                       // Set all chanels to servo mode
                SerBuf[1] = 0xA0;
                SerBuf[2] = 0x55;
                SerBuf[3] = 0x04;
                SerBuf[4] = 0x01;
                SerBuf[5] = 0x54;                                       // 84 mode bytes coming
                int x = 0;
                for (x = 6; x <= 90; x++)
                    SerBuf[x] = 0x19;                                   // Servo mode
                transmit(90);
                receive(1);
            }
            catch
            {
                MessageBox.Show("Error opening to serial port", "SD84", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                usb_found = 0;;
            }
        }

        private void servo_value_set(object sender, EventArgs e)
        {
            ushort value = Convert.ToUInt16(trackBar_servo1.Value); // Get the value to set from the track bar
            SerBuf[0] = 0xAA;                                       // Put the command together to set a servo position
            SerBuf[1] = 0xA0;
            SerBuf[2] = 0x55;
            SerBuf[3] = 0x01;
            SerBuf[4] = selected_servo;
            SerBuf[5] = 0x02;
            SerBuf[6] = (byte)(value & 0xFF);                       // Low byte of servo position
            SerBuf[7] = (byte)(value >> 8);                         // High byte of servo position
            transmit(8);
            receive(1);
        }

    }
}
